import random
from node import Node
from abstract_network import AbstractNetwork


class RandomNetwork(AbstractNetwork):
    """
    This network class inherits basic network functionality from the AbstractNetwork-class. It overwrites the
    __create_network__ function to create a random network.
    """
    
    def __create_network__(self, n_nodes, n_edges):
        """
        Creates a random network with the specified number of nodes and edges.
        1. build a list of n nodes
        2. create m edges between randomly selected nodes that are not yet connected
        (Hint: when adding an edge A --> B, do not forget to also add the edge B --> A)
        :param n_nodes: number of nodes
        :param n_edges: number of edges
        """
        # create n nodes and add them to the network
        for i in range(n_nodes):
            self.add_node(Node(i))

        # fully connect the network if n_edges is bigger than the possible number of edges
        if n_edges > (n_nodes * (n_nodes - 1)) / 2:
            for i in range(self.size()):
                for j in range(i + 1, self.size()):
                    self.add_edge_by_id(i, j)
            return

        # initialise the number of established edges
        i = 0
        
        # until all links are set...
        while i < n_edges:
            # ...pick two nodes
            node_1, node_2 = random.sample(list(self.nodes.values()), 2)

            # if there already exists an edge between the two nodes, choose another pair of nodes
            if node_1.has_edge_to(node_2) or node_2.has_edge_to(node_1):
                continue

            # add links two both nodes (since we're assuming an undirected graph)
            node_1.add_edge(node_2)
            node_2.add_edge(node_1)

            # increase the edge counter
            i += 1
