import matplotlib.pyplot as plt


def plot_distribution_comparison(histograms, legend, title):
    """
    Plots a list of histograms with matching list of descriptions as the legend. There is nothing to do here.
    :param histograms: list of histograms
    :param legend: list of matching histogram names
    :param title: plot title
    """
    # determine max. length
    longest = max(len(histogram) for histogram in histograms)
    
    # extend "shorter" distributions
    for histogram in histograms:
        histogram.extend([0.0] * (longest - len(histogram)))
        
    # plots histograms
    for histogram in histograms:
        plt.plot(range(len(histogram)), histogram, marker='x')
    
    # axis labels
    plt.xlabel('degree')
    plt.ylabel('p')
    
    # finish the plot
    plt.legend(legend)
    plt.title(title)
    plt.tight_layout()
    plt.show()


def plot_distribution_comparison_log(histograms, legend, title):
    """
    Plots a list of histograms with matching list of descriptions as the legend with logarithmic axes.
    There is nothing to do here.
    :param histograms: list of histograms
    :param legend: list of matching histogram names
    :param title: plot title
    """
    # set the axes to logarithmic scale
    ax = plt.subplot()
    ax.set_xscale('log')
    ax.set_yscale('log')

    # determine max. length
    longest = max(len(histogram) for histogram in histograms)

    # extend "shorter" distributions
    for histogram in histograms:
        histogram.extend([0.0] * (longest - len(histogram)))

    # plots histograms
    for histogram in histograms:
        ax.plot(range(len(histogram)), histogram, marker='x', linestyle='')

    # axis labels
    plt.xlabel('degree')
    plt.ylabel('p')

    # finish the plot
    plt.legend(legend)
    plt.title(title)
    plt.tight_layout()
    plt.show()


def scale_free_distribution(max_degree, gamma):
    """
    Generates a power law distribution histogram up to the maximum degree with slope gamma.
    :param max_degree: maximum degree
    :param gamma: slope
    :return: normalised power law histogram
    """
    # TODO
    raise NotImplementedError


def cumulative(dist):
    """
    Computes the cumulative distribution of a probabilistic distribution.
    :param dist: probabilistic distribution
    :return: cumulative distribution
    """
    # TODO
    raise NotImplementedError


def KS_dist(histogram_a, histogram_b):
    """
    Computes the Kolmogorov-Smirnov distance between two histograms.
    1. convert the histograms to cumulative distributions
    2. find the position where the cumulative distributions differ the most and return that distance
    :param histogram_a: first histogram
    :param histogram_b: second histogram
    :return: maximal distance
    """
    # TODO
    raise NotImplementedError
